/*
 * *****************************************************************************************
 *
 * 		Copyright (C) 2014-2021 Gowin Semiconductor Technology Co.,Ltd.
 * 		
 * @file			main.c
 * @author		Embedded Development Team
 * @version		V1.x.x
 * @date			2021-01-01 09:00:00
 * @brief			Main function.
 ******************************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "gw1ns4c.h"
#include <stdio.h>

/* Declarations ------------------------------------------------------------------*/
void UartInit(void);
void GPIOInit(void);
void delay_ms(__IO uint32_t delay_ms);

/* Functions ------------------------------------------------------------------*/
int main()
{
	NVIC_InitTypeDef InitTypeDef_NVIC;
	
  SystemInit();	//Initializes system
  UartInit();		//Initializes UART0
	GPIOInit();		//Initializes GPIO
	
	//Enable UART0 interrupt handler
	InitTypeDef_NVIC.NVIC_IRQChannel = UART0_IRQn;
  InitTypeDef_NVIC.NVIC_IRQChannelPreemptionPriority = 1;
  InitTypeDef_NVIC.NVIC_IRQChannelSubPriority = 1;
  InitTypeDef_NVIC.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&InitTypeDef_NVIC);
	
	printf("/************GOWINSEMI*****************/\r\n");
	printf("           GOWIN_EMPU_M1                \r\n");
	printf("/**************************************/\r\n");

  while(1);
}

//Initializes GPIO
void GPIOInit(void)
{
	GPIO_InitTypeDef GPIO_InitType;
	
	GPIO_InitType.GPIO_Pin = GPIO_Pin_0;
	GPIO_InitType.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitType.GPIO_Int = GPIO_Int_Disable;

	GPIO_Init(GPIO0,&GPIO_InitType);

  GPIO_SetBit(GPIO0,GPIO_Pin_0);
}

//Initialized UART0
void UartInit(void)
{
  UART_InitTypeDef UART_InitStruct;
	
  UART_InitStruct.UART_Mode.UARTMode_Tx = ENABLE;
  UART_InitStruct.UART_Mode.UARTMode_Rx = ENABLE;
  UART_InitStruct.UART_Int.UARTInt_Tx = DISABLE;
  UART_InitStruct.UART_Int.UARTInt_Rx = ENABLE;//Enable UART0 RX interrupt register
  UART_InitStruct.UART_Ovr.UARTOvr_Tx = DISABLE;
  UART_InitStruct.UART_Ovr.UARTOvr_Rx = DISABLE;
  UART_InitStruct.UART_Hstm = DISABLE;
  UART_InitStruct.UART_BaudRate = 115200;//Baud Rate
	
  UART_Init(UART0,&UART_InitStruct);
}

/**
  * @brief  This function handles UART0 interrupt request.
  * @param  none
  * @retval none
  */
void UART0_Handler(void)
{
	char num = '0';
	
	if(UART_GetRxIRQStatus(UART0) == SET)
	{
		num  = UART_ReceiveChar(UART0);
		printf("UART0 receives a Rx interrupt value %c\r\n",num);
		
		
		if(num == '1')
		{
			GPIO_ResetBit(GPIO0,GPIO_Pin_0);
			delay_ms(500);
			GPIO_SetBit(GPIO0,GPIO_Pin_0);
			delay_ms(500);
			
		}
		else if(num == '2')
		{
			for(int i = 0;i < 2;i++)
			{
				GPIO_ResetBit(GPIO0,GPIO_Pin_0);
				delay_ms(500);
				GPIO_SetBit(GPIO0,GPIO_Pin_0);
				delay_ms(500);
			}
		}
		else if(num == '3')
		{
			for(int i = 0;i < 3;i++)
			{
				GPIO_ResetBit(GPIO0,GPIO_Pin_0);
				delay_ms(500);
				GPIO_SetBit(GPIO0,GPIO_Pin_0);
				delay_ms(500);
			}
		}
		else if(num == '4')
		{
			for(int i = 0;i < 4;i++)
			{
				GPIO_ResetBit(GPIO0,GPIO_Pin_0);
				delay_ms(500);
				GPIO_SetBit(GPIO0,GPIO_Pin_0);
				delay_ms(500);
			}
		}
		else
		{
			printf("No operation!\r\n");
		}
	}
	
	UART_ClearRxIRQ(UART0);
}

//delay ms
void delay_ms(__IO uint32_t delay_ms)
{
	for(delay_ms=(SystemCoreClock>>13)*delay_ms; delay_ms != 0; delay_ms--);
}
